# muscle::AbstractReflectSession class [(API)](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractReflectSession.html)

```#include "reflector/AbstractReflectSession.h"```

[AbstractReflectSession](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractReflectSession.html) is an abstract base class for any object that wants to handle the needs of a single file descriptor in the [ReflectServer](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ReflectServer.html)'s event loop.

* Each session object holds an I/O gateway (via an [AbstractMessageIOGatewayRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractMessageIOGateway.html)) which it uses to convert incoming bytes from its socket/file-descriptor to Messages (and vice versa).  This gateway can be manually installed (via [SetGateway()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractReflectSession.html#a09ab9713237ce246d1277210d39b12a9)), or if not, a default gateway object will be automatically created by calling the session object's [CreateGateway()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractReflectSession.html#a9c73fb899e0c122d925751342568ef13) method.
* When the session is first attached to the [ReflectServer](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ReflectServer.html), the session's [AttachedToServer()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ServerComponent.html#acd034f5c7d067f473fe2f8df182e4f91) method will be called.  In this method the session can do any setup/initialization work it wants to do, now that it has access to the [ReflectServer](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ReflectServer.html)'s resources.
* Each Message generated by the gateway object's parser (in response to incoming data) is passed to the session via a call to the session's [MessageReceivedFromGateway()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractGatewayMessageReceiver.html#a8c1bf1beef34ff908a089fb0331d0771) method (which must be implemented by the session subclass)
* When a Message is handed over to the session by one of the other sessions living on the same [ReflectServer](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ReflectServer.html), [MessageReceivedFromSession()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ServerComponent.html#a077bf3e532fa0760b2ce3f0650f27d82) will be called.
* When the session wants to send a Message back to its own client (via the gateway), it can call [AddOutgoingMessage())](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractReflectSession.html#a3d309230362a0d2c6960e782e82e5866) to do so.
* When the session's TCP connection is closed (e.g. the client-software associated wih this session has quit or disconnected), the method's [ClientConnectionClosed()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractReflectSession.html#a7b3ab46167ab8e0f816588913b3af987) method is called.  The default implementation of this method returns true, indicating that the session object should be detached from the server and destroyed ASAP.
* If the session wants to commit suicide, it can call [EndSession()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1AbstractReflectSession.html#a70cf9ddd7b1758561710af7c7ebc3411) on itself and it will be detached and destroyed ASAP.
* Just before the session is detached from the [ReflectServer](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ReflectServer.html), its [AboutToDetachFromServer()](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ServerComponent.html#a03a3d3aa94ede5f54ba5c68ca0397f2a) method will be called.  This gives the session a final chance to say its goodbyes, while it still has access to the [ReflectServer](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ReflectServer.html)'s resources.
* Included concrete subclasses include [DumbReflectSession](https://public.msli.com/lcs/muscle/html/classmuscle_1_1DumbReflectSession.html) (which simply forwards incoming Messages from the session's gateway to all other connected sessions) and [StorageReflectSession](https://public.msli.com/lcs/muscle/html/classmuscle_1_1StorageReflectSession.html) (which implements the full MUSCLE node-tree-storage and regex-based forwarding logic as described [here](https://public.msli.com/lcs/muscle/muscle/html/Beginners%20Guide.html))

Try compiling and running the mini-example-programs in `muscle/html/muscle-by-example/examples/reflector` (enter `make` to compile example_*, and then run each from Terminal while looking at the corresponding .cpp file)

Quick links to source code of relevant MUSCLE-by-example programs:

* [reflector/example_1_dumb_server.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/reflector/example_1_dumb_server.cpp)
* [reflector/example_2_dumb_client.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/reflector/example_2_dumb_client.cpp)
* [reflector/example_3_annotated_dumb_server.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/reflector/example_3_annotated_dumb_server.cpp)
* [reflector/example_4_smart_server.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/reflector/example_4_smart_server.cpp)
* [reflector/example_5_smart_client.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/reflector/example_5_smart_client.cpp)
* [reflector/example_6_smart_server_with_pulsenode.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/reflector/example_6_smart_server_with_pulsenode.cpp)
* [reflector/example_7_smart_server_with_udp_pingpong.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/reflector/example_7_smart_server_with_udp_pingpong.cpp)
